export interface BEMJoinOptions {
    /**
     * The separator used between the BEM block and element name.
     * The default is "__" (e.g., foo__bar).
     */
    elementSeparator?: string;
    /**
     * The separator used between the BEM element and modifier names.
     * The default is "--" (e.g., foo__bar--baz).
     */
    modifierSeparator?: string;
}
/**
 * Returns a function that can be used to construct BEM class names.
 */
export declare function bemJoin(
/**
 * The BEM block name on the left side of each join.
 */
blockName: string): ConstructClassNames;
export declare function bemJoin(options?: BEMJoinOptions): CurryBlockName;
/**
 * BEM modifiers for blocks and elements.
 */
export declare type BEMModifiers = Record<string, boolean | undefined>;
export declare type CurryBlockName = (
/**
 * The BEM block name on the left side of each join.
 */
blockName: string) => ConstructClassNames;
export interface ConstructClassNames {
    (
    /**
     * If specified, the first class name returned will be the BEM block and element names joined (e.g., foo__bar).
     */
    element: string, 
    /**
     * If specified, the last class names returned will be the joined BEM block and element followed by any number of modifiers provided (e.g., foo__bar--mod1 foo__bar--mod2)
     */
    elementModifiers?: BEMModifiers): string;
    (
    /**
     * If specified, the last class names returned will be the BEM block name followed by any number of modifiers provided (e.g., foo--mod1 foo--mod2).
     */
    blockModifiers?: BEMModifiers): string;
}
